package ch.papyro.flutter_verifai;

import android.app.Activity;
import android.content.res.Resources;
import android.util.Log;

import androidx.annotation.NonNull;

import com.verifai.core.Verifai;
import com.verifai.core.VerifaiConfiguration;
import com.verifai.core.listeners.VerifaiResultListener;
import com.verifai.core.result.VerifaiResult;

import org.jetbrains.annotations.NotNull;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.Result;

final class MethodCallHandlerImpl implements MethodChannel.MethodCallHandler {
    private final Activity activity;
    private final MethodChannel methodChannel;

    MethodCallHandlerImpl(
            Activity activity,
            BinaryMessenger messenger) {
        this.activity = activity;
        methodChannel = new MethodChannel(messenger, "flutter_verifai");
        methodChannel.setMethodCallHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
        switch (call.method) {
            case "initializeVerifai":

                String licence = call.argument("licence");
                assert licence != null;
                Verifai.setLicence(activity, licence); //

                VerifaiConfiguration configuration = new VerifaiConfiguration();
                configuration.setScanDuration(5.0);
                Verifai.configure(configuration);
                VerifaiResultListener resultListener = new VerifaiResultListener() {
                    @Override
                    public void onCanceled() {
                        Log.println(Log.ERROR, "flutter_verifai", "canceled");
                    }

                    @Override
                    public void onSuccess(VerifaiResult verifaiResult) {
                        if(verifaiResult.mrzData != null) {
                            Map<String, Object> resultMap = new HashMap<String,Object>();
                            if(verifaiResult.mrzData.getFirstName() != null && verifaiResult.mrzData.getLastName() != null && verifaiResult.mrzData.getDateOfBirth() != null
                            && verifaiResult.mrzData.getMrzString() != null && verifaiResult.mrzData.getSex() != null && verifaiResult.mrzData.getCountryCode() != null
                                    && verifaiResult.mrzData.getDateOfExpiry() != null) {
                                resultMap.put("firstName", verifaiResult.mrzData.getFirstName());
                                resultMap.put("lastName", verifaiResult.mrzData.getLastName());
                                resultMap.put("dateOfBirth", verifaiResult.mrzData.getDateOfBirth());
                                resultMap.put("idCode",  verifaiResult.mrzData.getMrzString());
                                resultMap.put("sex", verifaiResult.mrzData.getSex());
                                resultMap.put("country", verifaiResult.mrzData.getCountryCode());
                                resultMap.put("dateOfExpiry", verifaiResult.mrzData.getDateOfExpiry());
                                result.success(resultMap);

                            } else {
                                Log.println(Log.ERROR, "flutter_verifai", "Data invalid, something is missing");
                                result.error("VERIFICATION_INVALID_DATA", "Data invalid, something is missing", "Some data are missing from the verifai extraction");
                            }
                        }
                    }

                    @Override
                    public void onError(@NotNull Throwable throwable) {
                        Log.d("error", throwable.getMessage());
                    }
                };
                Verifai.startScan(activity, resultListener);// The licence string that has been obtained from the dashboard.
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    void stopListening() {
        methodChannel.setMethodCallHandler(null);
    }
}
